import java.io.*;
import java.net.Socket;
import java.nio.file.*;
import java.text.SimpleDateFormat;
import java.util.TimeZone;
import java.util.concurrent.*;

public class HttpHandler implements Runnable {
    private final Socket socket;
    private final String rootPath;

    public HttpHandler(Socket socket, String rootPath) {
        this.rootPath = rootPath;
        this.socket = socket;
    }

    public void run() {
        try {
            handleRequest();
        } catch (Exception e) {
            System.err.println("Error Occured: " + e.getMessage());
            try {
                socket.close();
                System.exit(0);
            } catch (IOException e1) {
                System.err.println("Error Closing socket Connection.");
                System.exit(0);
            }
            System.err.println("Server is Terminating!");
        }
    }

    private void handleRequest() {
        ScheduledExecutorService executor = Executors.newSingleThreadScheduledExecutor();

        Runnable task = () -> {
            InputStream input;
            OutputStream output;
            File root = new File(rootPath);

            if (root.isDirectory()) {
                try {
                    input = socket.getInputStream();
                    output = socket.getOutputStream();
                    serverRequest(input, output, root.toString());
                    output.close();
                    input.close();
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
            try {
                socket.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        };
        executor.schedule(task, 1, TimeUnit.SECONDS);
        executor.shutdown();
    }

    private void serverRequest(InputStream input, OutputStream output,
                               String root) throws Exception {
        String line;
        BufferedReader bf = new BufferedReader(new InputStreamReader(input));
        while ((line = bf.readLine()) != null) {
            if (line.length() <= 0) {
                break;
            }
            if (line.startsWith("GET")) {
                String filename = line.split(" ")[1].substring(1);
                if (filename.isBlank()) {
                    filename = "index.html";
                }
                File resource = new File(root + File.separator + filename);
                if (resource.isFile()) {
                    populateResponse(resource, output);
                } else {
                    String Content_NOT_FOUND = "<html><head></head><body><h1>File Not Found</h1></body></html>";

                    String REQ_NOT_FOUND = "HTTP/1.1 404 Not Found\n\n";
                    String header = REQ_NOT_FOUND + Content_NOT_FOUND;

                    output.write(header.getBytes());
                }
                break;
            }
        }
    }

    private void populateResponse(File resource, OutputStream output)
            throws IOException {
        SimpleDateFormat format = new SimpleDateFormat(
                "EEE, d MMM yyyy HH:mm:ss z");
        format.setTimeZone(TimeZone.getTimeZone("GMT"));

        String REQ_FOUND = "HTTP/1.0 200 OK\n";
        String SERVER = "Server: HTTP server/0.1\n";
        String DATE = "Date: " + format.format(new java.util.Date()) + "\n";
        String CONTENT_TYPE = "Content-type: text/html\n";
        String LENGTH = "Content-Length: " + (resource.length()) + "\n\n";

        String header = REQ_FOUND + SERVER + DATE + CONTENT_TYPE + LENGTH;
        output.write(header.getBytes());

        Files.copy(Paths.get(resource.toString()), output);
        output.flush();
    }
}
