import org.junit.*;

import java.io.*;
import java.net.*;
import java.util.*;

import static org.junit.Assert.*;

public class WorldCupCrawlerSampleTest {
    private static Server server;
    private static Thread serverThread;
    private static final Map<String, Stadium> stadiums = new HashMap<>();

    @BeforeClass
    public static void init() throws Exception {
        server = new Server(80, "/home/project/src/test/java/www");
        serverThread = new Thread(() -> {
            try {
                server.start();
            } catch (IOException e) {
                e.printStackTrace();
            }
        });
        serverThread.start();

        stadiums.put("Al-Bayt Stadium",
                new Stadium("Al-Bayt Stadium", new Country("Qatar"), 60000));
        stadiums.put("Al-Thumama Stadium",
                new Stadium("Al-Thumama Stadium", new Country("Qatar"), 44740));
        stadiums.put("Khalifa International Stadium",
                new Stadium("Khalifa International Stadium",
                        new Country("Qatar"), 45416));
        stadiums.put("Ahmad bin Ali Stadium",
                new Stadium("Ahmad bin Ali Stadium", new Country("Qatar"),
                        40000));
        stadiums.put("Lusail Iconic Stadium",
                new Stadium("Lusail Iconic Stadium", new Country("Qatar"),
                        80000));
        stadiums.put("Stadium 974",
                new Stadium("Stadium 974", new Country("Qatar"),
                        40000));
        stadiums.put("Al-Janoub Stadium",
                new Stadium("Al-Janoub Stadium", new Country("Qatar"),
                        40000));
        stadiums.put("Education City Stadium",
                new Stadium("Education City Stadium", new Country("Qatar"),
                        45350));

        Thread.sleep(3000);
    }

    @AfterClass
    public static void clean() throws IOException {
        serverThread.stop();
        server.serverSocket.close();
    }

    @Test(timeout = 5000)
    public void testGetStadiumReturnsCorrectMatches() throws Exception {
        assertEquals(stadiums.get("Al-Bayt Stadium"),
                WorldCupCrawler.getStadium(
                        new URL("http://localhost/al-bayt-stadium.html")));
    }

    @Test(timeout = 12000)
    public void testGetMatchesReturnsCorrectMatches() throws Exception {
        testGetMatches();
    }

    @Test(timeout = 4000)
    public void testGetMatchesShouldUseMultithreading() throws Exception {
        testGetMatches();
    }

    private void testGetMatches() throws Exception {
        List<Match> expectedMatches = new ArrayList<>();

        expectedMatches.add(
                new Match(new Date(1668960000000L), "A", new Team("Qatar"),
                        new Team("Ecuador"), stadiums.get("Al-Bayt Stadium")));

        expectedMatches.add(
                new Match(new Date(1669046400000L), "A", new Team("Senegal"),
                        new Team("Netherlands"),
                        stadiums.get("Al-Thumama Stadium")));

        expectedMatches.add(
                new Match(new Date(1669035600000L), "B", new Team("England"),
                        new Team("Iran"),
                        stadiums.get("Khalifa International Stadium")));

        expectedMatches.add(
                new Match(new Date(1669057200000L), "B",
                        new Team("United States"),
                        new Team("Wales"),
                        stadiums.get("Ahmad bin Ali Stadium")));

        expectedMatches.add(
                new Match(new Date(1669111200000L), "C", new Team("Argentina"),
                        new Team("Saudi Arabia"),
                        stadiums.get("Lusail Iconic Stadium")));

        expectedMatches.add(
                new Match(new Date(1669132800000L), "C", new Team("Mexico"),
                        new Team("Poland"), stadiums.get("Stadium 974")));

        expectedMatches.add(
                new Match(new Date(1669143600000L), "D", new Team("France"),
                        new Team("Australia"),
                        stadiums.get("Al-Janoub Stadium")));

        expectedMatches.add(
                new Match(new Date(1669122000000L), "D", new Team("Denmark"),
                        new Team("Tunisia"),
                        stadiums.get("Education City Stadium")));

        expectedMatches.add(
                new Match(new Date(1669219200000L), "E", new Team("Spain"),
                        new Team("Costa Rica"),
                        stadiums.get("Al-Thumama Stadium")));

        WorldCupCrawler crawler = new WorldCupCrawler(
                new URL("http://localhost"));
        List<Match> extractedMatches = crawler.getMatches();

        assertEquals(expectedMatches, extractedMatches);
    }
}